package com.zy.omp.controller.wx;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.zy.omp.common.ServerResponse;
import com.zy.omp.common.controller.BaseController;
import com.zy.omp.model.Device;
import com.zy.omp.model.Relation;
import com.zy.omp.service.DeviceInfoStateService;
import com.zy.omp.service.DeviceService;
import com.zy.omp.service.RelationService;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * 设备 Controller
 *
 * @author yang xiao kun
 * create on 2021/10/18
 */
@RestController
@RequestMapping("omp/api/wx/device")
public class DeviceController extends BaseController {

    @Resource
    private DeviceInfoStateService deviceInfoStateService;
    @Resource
    private RelationService relationService;
    @Resource
    private DeviceService deviceService;

    /**
     * 查询用户默认设备的最新数据
     */
    @GetMapping("getLatestState.do")
    public ServerResponse getLatestState() {
        return ServerResponse.createBySuccess(deviceInfoStateService.getLatestState(getOpenId()));
    }

    /**
     * 查询用户默认设备
     */
    @GetMapping("getDefaultDevice.do")
    public ServerResponse getDefaultDevice() {
        return ServerResponse.createBySuccess(relationService.getDefaultByOpenId(getOpenId()));
    }

    /**
     * 查询用户所有的设备
     */
    @GetMapping("getListByOpenId.do")
    public ServerResponse getListByOpenId() {
        return ServerResponse.createBySuccess(relationService.getListByOpenId(getOpenId()));
    }

    /**
     * 设置默认设备
     */
    @PostMapping("setDefault.do")
    public ServerResponse setDefault(String openNum) {
        relationService.update(new UpdateWrapper<Relation>()
                .set("isDefault", 1)
                .eq("openId", getOpenId())
                .eq("openNum", openNum));
        return ServerResponse.createBySuccess();
    }

    /**
     * 更新设备名称
     */
    @PostMapping("updateName.do")
    public ServerResponse updateName(Relation relation) {
        relationService.update(new UpdateWrapper<Relation>()
                .set("deviceName", relation.getDeviceName())
                .eq("openId", getOpenId())
                .eq("openNum", relation.getOpenNum()));
        return ServerResponse.createBySuccess();
    }

    /**
     * 添加绑定关系
     */
    @PostMapping("bind.do")
    public ServerResponse bind(Relation relation) {
        //校验设备码有效性
        if (deviceService.getOne(new QueryWrapper<Device>().eq("openNum", relation.getOpenNum())) == null)
            return ServerResponse.createByWarning("设备码无效!");
        //校验是否有绑定关系
        if (relationService.checkBind(relation.getOpenNum())) return ServerResponse.createByWarning("该设备已经有用户绑定!");
        relation.setOpenId(getOpenId());
        relationService.save(relation);
        return ServerResponse.createBySuccess();
    }

    /**
     * 解除绑定关系
     */
    @PostMapping("unbind.do")
    public ServerResponse unbind(String openNum) {
        relationService.remove(new QueryWrapper<Relation>()
                .eq("openNum", openNum)
                .eq("openId", getOpenId()));
        return ServerResponse.createBySuccess();
    }

    /**
     * 修改默认设备
     * 即登陆以后使用那个设备
     *
     * @param openNum 设备码
     */
    @PostMapping("changeDefault.do")
    public ServerResponse changeDefault(String openNum) {
        //删除上一个默认设备
        relationService.update(Wrappers.<Relation>lambdaUpdate().eq(Relation::getIsDefault, 1)
                .eq(Relation::getOpenId, getOpenId()).set(Relation::getIsDefault, 0));
        //添加新的默认设备
        relationService.update(Wrappers.<Relation>lambdaUpdate().eq(Relation::getOpenNum, openNum)
                .set(Relation::getIsDefault, 1));
        return ServerResponse.createBySuccess();
    }
}
